/**
 * Copyright (c) 2016-present Invertase Limited & Contributors
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this library except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */

#import <Firebase/Firebase.h>
#import <React/RCTLog.h>
#import "FirebaseCrashlyticsInitProvider.h"
#import "FirebasePreferences.h"

#if __has_include(<FirebaseCoreExtension/FIRAppInternal.h>)
#import <FirebaseCoreExtension/FIRAppInternal.h>
#else
#import <FirebaseCore/FIRAppInternal.h>
#endif

NSString *const KEY_CRASHLYTICS_DEBUG_ENABLED = @"crashlytics_debug_enabled";
NSString *const KEY_CRASHLYTICS_AUTO_COLLECTION_ENABLED = @"crashlytics_auto_collection_enabled";
NSString *const KEY_CRASHLYTICS_IS_ERROR_GENERATION_ON_JS_CRASH_ENABLED = @"crashlytics_is_error_generation_on_js_crash_enabled";
NSString *const KEY_CRASHLYTICS_JAVASCRIPT_EXCEPTION_HANDLER_CHAINING_ENABLED = @"crashlytics_javascript_exception_handler_chaining_enabled";

static NSString *const FirebaseDomainIdentifier = @"io.invertase.firebase";

@implementation FirebaseCrashlyticsInitProvider

+ (void)load {
  [FIRApp registerInternalLibrary:self withName:@"react-native-firebase-crashlytics" withVersion:@"11.5.0"];
}

+ (BOOL)isCrashlyticsCollectionEnabled {
  BOOL enabled = YES;

  NSDictionary *infoPList = [[NSBundle mainBundle] infoDictionary];  

  if ([[FirebasePreferences shared] contains:KEY_CRASHLYTICS_AUTO_COLLECTION_ENABLED]) {
    enabled = [[FirebasePreferences shared] getBooleanValue:KEY_CRASHLYTICS_AUTO_COLLECTION_ENABLED defaultValue:YES];
    RCTLog(@"FirebaseCrashlyticsInit isCrashlyticsCollectionEnabled via User Defaults: %d", enabled);
  } else if ([infoPList objectForKey:KEY_CRASHLYTICS_AUTO_COLLECTION_ENABLED] != nil) {
    enabled = [[infoPList valueForKey:KEY_CRASHLYTICS_AUTO_COLLECTION_ENABLED] boolValue];
    RCTLog(@"FirebaseCrashlyticsInit isCrashlyticsCollectionEnabled via Info.plist: %d", enabled);
  }
  
#ifdef DEBUG
  enabled = NO;
  if ([infoPList objectForKey:KEY_CRASHLYTICS_DEBUG_ENABLED] != nil) {
     enabled = [[infoPList valueForKey:KEY_CRASHLYTICS_DEBUG_ENABLED] boolValue];
  }
  RCTLog(@"FirebaseCrashlyticsInit isCrashlyticsCollectionEnabled after checking Info.plist %@: %d", KEY_CRASHLYTICS_DEBUG_ENABLED, enabled);
#endif

  RCTLog(@"FirebaseCrashlyticsInit isCrashlyticsCollectionEnabled final value: %d", enabled);

  return enabled;
}

+ (BOOL)isErrorGenerationOnJSCrashEnabled {
  BOOL enabled = YES;

  if ([[FirebasePreferences shared] contains:KEY_CRASHLYTICS_IS_ERROR_GENERATION_ON_JS_CRASH_ENABLED]) {
    enabled = [[FirebasePreferences shared] getBooleanValue:KEY_CRASHLYTICS_IS_ERROR_GENERATION_ON_JS_CRASH_ENABLED defaultValue:YES];
    RCTLog(@"FirebaseCrashlyticsInit isErrorGenerationOnJSCrashEnabled via User Defaults: %d", enabled);
  } else {
    NSDictionary *infoPList = [[NSBundle mainBundle] infoDictionary];  
    if ([infoPList objectForKey:KEY_CRASHLYTICS_IS_ERROR_GENERATION_ON_JS_CRASH_ENABLED] != nil) {
        enabled = [[infoPList valueForKey:KEY_CRASHLYTICS_IS_ERROR_GENERATION_ON_JS_CRASH_ENABLED] boolValue];
        RCTLog(@"FirebaseCrashlyticsInit isErrorGenerationOnJSCrashEnabled via Info.plist: %d", enabled);
    }
  }

  RCTLog(@"FirebaseCrashlyticsInit isErrorGenerationOnJSCrashEnabled final value: %d", enabled);

  return enabled;
}

+ (BOOL)isCrashlyticsJavascriptExceptionHandlerChainingEnabled {
  BOOL enabled = YES;

  if ([[FirebasePreferences shared] contains:KEY_CRASHLYTICS_JAVASCRIPT_EXCEPTION_HANDLER_CHAINING_ENABLED]) {
    enabled = [[FirebasePreferences shared] getBooleanValue:KEY_CRASHLYTICS_JAVASCRIPT_EXCEPTION_HANDLER_CHAINING_ENABLED defaultValue:YES];
    RCTLog(@"FirebaseCrashlyticsInit isCrashlyticsJavascriptExceptionHandlerChainingEnabled via User Defaults: %d", enabled);
  } else {
    NSDictionary *infoPList = [[NSBundle mainBundle] infoDictionary];  
    if ([infoPList objectForKey:KEY_CRASHLYTICS_JAVASCRIPT_EXCEPTION_HANDLER_CHAINING_ENABLED] != nil) {
        enabled = [[infoPList valueForKey:KEY_CRASHLYTICS_JAVASCRIPT_EXCEPTION_HANDLER_CHAINING_ENABLED] boolValue];
        RCTLog(@"FirebaseCrashlyticsInit isCrashlyticsJavascriptExceptionHandlerChainingEnabled via Info.plist: %d", enabled);
    }
  }

  RCTLog(@"FirebaseCrashlyticsInit isCrashlyticsJavascriptExceptionHandlerChainingEnabled final value: %d", enabled);

  return enabled;
}

+ (NSArray<FIRComponent *> *)componentsToRegister {
  return @[];
}

/*
 * configureWithApp is automatically invoked by Firebase as this app is a registered FIRLibrary with the SDK.
 *
 * At this point "configure" has already been called on the FIRApp instance. This behavior is meant to mirror
 * what is done in ReactNativeFirebaseCrashlyticsInitProvider.java
 *
 * This pattern may not be supported long term https://github.com/firebase/firebase-ios-sdk/issues/2933
 */
+ (void)configureWithApp:(FIRApp *)app {
  // This setting is sticky. setCrashlyticsCollectionEnabled persists the setting to disk until it is explicitly set otherwise or the app is deleted.
  // Jump to the setCrashlyticsCollectionEnabled definition to see implementation details.
  [[FIRCrashlytics crashlytics] setCrashlyticsCollectionEnabled:self.isCrashlyticsCollectionEnabled];
  RCTLog(@"FirebaseCrashlyticsInit initialization successful");
}

@end
