/* eslint-disable no-console */
/*
 * Copyright (c) 2016-present Invertase Limited & Contributors
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this library except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */

import { NativeModules } from "react-native";
import StackTrace from "stacktrace-js";
import { createNativeErrorObj, setGlobalErrorHandler, setOnUnhandledPromiseRejectionHandler } from "./handlers";
import { isBoolean, isError, isObject, isString } from "./utils/validators";
const Native = NativeModules.FirebaseCrashlytics;
class FirebaseCrashlytics {
  constructor() {
    setGlobalErrorHandler(Native, this.invokeJSCallback);
    setOnUnhandledPromiseRejectionHandler(Native);
    this._isCrashlyticsCollectionEnabled = Native.isCrashlyticsCollectionEnabled;
    this._jsCallback = undefined;
  }
  get isCrashlyticsCollectionEnabled() {
    return this._isCrashlyticsCollectionEnabled;
  }
  async invokeJSCallback(...args) {
    if (typeof this._jsCallback === "function") {
      await this._jsCallback(...args);
    } else {
      console.warn(`FirebaseCrashlytics - no js callback provided`);
    }
  }
  setJSCallback(callback) {
    this._jsCallback = callback;
  }
  checkForUnsentReports() {
    if (this.isCrashlyticsCollectionEnabled) {
      throw new Error("crashlytics().setCrashlyticsCollectionEnabled(*) has been set to 'true', all reports are automatically sent.");
    }
    return Native.checkForUnsentReports();
  }
  crash() {
    Native.crash();
  }
  async deleteUnsentReports() {
    await Native.deleteUnsentReports();
  }
  didCrashOnPreviousExecution() {
    return Native.didCrashOnPreviousExecution();
  }
  log(message) {
    Native.log(`${message}`);
  }
  setAttribute(name, value) {
    if (!isString(name)) {
      throw new Error("crashlytics().setAttribute(*, _): The supplied property name must be a string.");
    }
    if (!isString(value)) {
      throw new Error("crashlytics().setAttribute(_, *): The supplied property value must be a string value.");
    }
    return Native.setAttribute(name, value);
  }
  setAttributes(object) {
    if (!isObject(object)) {
      throw new Error("crashlytics().setAttributes(*): The supplied arg must be an object of key value strings.");
    }
    return Native.setAttributes(object);
  }
  setUserId(userId) {
    if (!isString(userId)) {
      throw new Error("firebase.crashlytics().setUserId(*): The supplied userId must be a string value.");
    }
    return Native.setUserId(userId);
  }
  recordError(error, jsErrorName) {
    if (isError(error)) {
      StackTrace.fromError(error, {
        offline: true
      }).then(stackFrames => {
        Native.recordError(createNativeErrorObj(error, stackFrames, false, jsErrorName));
      });
    } else {
      console.warn("crashlytics().recordError(*) expects an instance of Error. Non Errors will be ignored.");
    }
  }
  sendUnsentReports() {
    if (this.isCrashlyticsCollectionEnabled) {
      Native.sendUnsentReports();
    }
  }
  setCrashlyticsCollectionEnabled(enabled) {
    if (!isBoolean(enabled)) {
      throw new Error("crashlytics().setCrashlyticsCollectionEnabled(*) 'enabled' must be a boolean.");
    }
    this._isCrashlyticsCollectionEnabled = enabled;
    return Native.setCrashlyticsCollectionEnabled(enabled);
  }
}
let instance;
const crashlytics = () => {
  if (!instance) {
    instance = new FirebaseCrashlytics();
  }
  return instance;
};
crashlytics();
export default crashlytics;
//# sourceMappingURL=index.js.map